/* $Id: algo_auth.c,v 1.1 2002/01/24 15:15:16 levitte Exp $ */

/* ====================================================================
 * Copyright (c) 1999 Levitte Programming.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *      This product includes software developed by Levitte Programming
 *      and its contributors.
 *
 * 4. The names "RLCRYPTO" and "Levitte Programming" or the names of its
 *    contributors must not be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *    For written permission, please contact levitte@lp.se.
 *
 * THIS SOFTWARE IS PROVIDED BY LEVITTE PROGRAMMING AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL LEVITTE
 * PROGRAMMING OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 */

/* Functions to create new algorithms.
   You're supposed to create a method structure with SYM_create(), alter
   the values with the SYM_set_*() functions and finally lock all those
   values into the structure with SYM_lock_values().  When the algorithm
   isn't needed any more, it can be discarded with SYM_destroy(). */
SYM_METHOD *SYM_create(void);
int SYM_set_name(SYM_METHOD *meth, const char *name);
int SYM_set_nid(SYM_METHOD *meth, int nid);
int SYM_set_keylen(SYM_METHOD *meth, size_t keylen);
int SYM_set_blocklen(SYM_METHOD *meth, size_t blocklen);
int SYM_set_init_fn(SYM_METHOD *meth,
		    int (*fn)(ALGO_CTX *, const char *, size_t));
int SYM_set_enc_dec_fn(SYM_METHOD *meth,
		       int (*fn)(ALGO_CTX *,
				 const char *, char *, size_t,
				 int));
int SYM_set_alloc_fn(SYM_METHOD *meth, int (*fn)(void));
int SYM_set_free_fn(SYM_METHOD *meth, int (*fn)(ALGO_CTX *));
int SYM_lock_values(SYM_METHOD *meth);
void SYM_destroy(SYM_METHOD *meth);

/* Functions to create new modes.
   You're supposed to create a method structure with MODE_create(), alter
   the values with the MODE_set_*() functions and finally lock all those
   values into the structure with MODE_lock_values().  When the algorithm
   isn't needed any more, it can be discarded with MODE_destroy(). */
MODE_METHOD *MODE_create(void);
int MODE_set_name(MODE_METHOD *meth, const char *name);
int MODE_set_nid(MODE_METHOD *meth, int nid);
int MODE_set_keylen(MODE_METHOD *meth, size_t keylen);
int MODE_set_blocklen(MODE_METHOD *meth, size_t blocklen);
int MODE_set_init_fn(MODE_METHOD *meth, int (*fn)(ALGO_CTX *, const char *));
int MODE_set_enc_dec_fn(MODE_METHOD *meth,
			int (*fn)(ALGO_CTX *,
				  const char *, char *, size_t,
				  int));
int MODE_set_alloc_fn(MODE_METHOD *meth, int (*fn)(void));
int MODE_set_free_fn(MODE_METHOD *meth, int (*fn)(ALGO_CTX *));
int MODE_lock_values(MODE_METHOD *meth);
void MODE_destroy(MODE_METHOD *meth);
